# coding=utf-8
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._client import OpenMeterClient  # type: ignore
from ._version import VERSION

__version__ = VERSION

try:
    from ._patch import __all__ as _patch_all
    from ._patch import *
except ImportError:
    _patch_all = []
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "OpenMeterClient",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore

_patch_sdk()



# coding=utf-8
# pylint: disable=wrong-import-position

"""
Re-exports all operations from the generated code for a cleaner API surface.
"""

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .._generated.operations._patch import *  # pylint: disable=unused-wildcard-import

# Re-export all operations from _generated.operations
from .._generated.operations._operations import *  # noqa: F401, F403

try:
    from .._generated.operations._patch import __all__ as _patch_all
    from .._generated.operations._patch import *  # noqa: F401, F403
except ImportError:
    _patch_all = []
from .._generated.operations._patch import patch_sdk as _patch_sdk

# Import and re-export the __all__ list
from .._generated.operations import __all__

__all__ = __all__
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore

_patch_sdk()



# coding=utf-8
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import PortalOperations  # type: ignore
from ._operations import AppsOperations  # type: ignore
from ._operations import AppStripeOperations  # type: ignore
from ._operations import CustomerAppsOperations  # type: ignore
from ._operations import CustomersOperations  # type: ignore
from ._operations import FeaturesOperations  # type: ignore
from ._operations import PlansOperations  # type: ignore
from ._operations import PlanAddonsOperations  # type: ignore
from ._operations import AddonsOperations  # type: ignore
from ._operations import SubscriptionsOperations  # type: ignore
from ._operations import SubscriptionAddonsOperations  # type: ignore
from ._operations import EntitlementsOperations  # type: ignore
from ._operations import GrantsOperations  # type: ignore
from ._operations import SubjectsOperations  # type: ignore
from ._operations import CustomerOperations  # type: ignore
from ._operations import CustomerEntitlementOperations  # type: ignore
from ._operations import CustomerStripeOperations  # type: ignore
from ._operations import MarketplaceOperations  # type: ignore
from ._operations import AppCustomInvoicingOperations  # type: ignore
from ._operations import EventsOperations  # type: ignore
from ._operations import EventsV2Operations  # type: ignore
from ._operations import MetersOperations  # type: ignore
from ._operations import SubjectsOperations  # type: ignore
from ._operations import DebugOperations  # type: ignore
from ._operations import NotificationChannelsOperations  # type: ignore
from ._operations import NotificationRulesOperations  # type: ignore
from ._operations import NotificationEventsOperations  # type: ignore
from ._operations import EntitlementsV2Operations  # type: ignore
from ._operations import CustomerEntitlementsV2Operations  # type: ignore
from ._operations import CustomerEntitlementV2Operations  # type: ignore
from ._operations import GrantsV2Operations  # type: ignore
from ._operations import BillingProfilesOperations  # type: ignore
from ._operations import CustomerOverridesOperations  # type: ignore
from ._operations import InvoicesOperations  # type: ignore
from ._operations import InvoiceOperations  # type: ignore
from ._operations import CustomerInvoiceOperations  # type: ignore
from ._operations import ProgressOperations  # type: ignore
from ._operations import CurrenciesOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "PortalOperations",
    "AppsOperations",
    "AppStripeOperations",
    "CustomerAppsOperations",
    "CustomersOperations",
    "FeaturesOperations",
    "PlansOperations",
    "PlanAddonsOperations",
    "AddonsOperations",
    "SubscriptionsOperations",
    "SubscriptionAddonsOperations",
    "EntitlementsOperations",
    "GrantsOperations",
    "SubjectsOperations",
    "CustomerOperations",
    "CustomerEntitlementOperations",
    "CustomerStripeOperations",
    "MarketplaceOperations",
    "AppCustomInvoicingOperations",
    "EventsOperations",
    "EventsV2Operations",
    "MetersOperations",
    "SubjectsOperations",
    "DebugOperations",
    "NotificationChannelsOperations",
    "NotificationRulesOperations",
    "NotificationEventsOperations",
    "EntitlementsV2Operations",
    "CustomerEntitlementsV2Operations",
    "CustomerEntitlementV2Operations",
    "GrantsV2Operations",
    "BillingProfilesOperations",
    "CustomerOverridesOperations",
    "InvoicesOperations",
    "InvoiceOperations",
    "CustomerInvoiceOperations",
    "ProgressOperations",
    "CurrenciesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8

from typing import Any, Optional
from typing_extensions import Self

from corehttp.credentials import ServiceKeyCredential
from corehttp.runtime import policies

from ._generated._client import OpenMeterClient


class Client(OpenMeterClient):
    def __init__(
        self,
        endpoint: str = "https://openmeter.cloud",
        token: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        if token and not kwargs.get("authentication_policy"):
            credential = ServiceKeyCredential(token)
            kwargs["authentication_policy"] = policies.ServiceKeyCredentialPolicy(
                credential, "Authorization", prefix="Bearer"
            )

        super().__init__(endpoint=endpoint, **kwargs)

    def __enter__(self) -> Self:
        return super().__enter__()

    def __exit__(self, *exc_details: Any) -> None:
        return super().__exit__(*exc_details)



# coding=utf-8

from typing import Any

from corehttp.runtime import policies

from ._version import VERSION


class OpenMeterClientConfiguration:
    """Configuration for OpenMeterClient.

    Note that all parameters used to create this instance are saved as instance
    attributes.

    :param endpoint: Service host. Default value is "https://127.0.0.1".
    :type endpoint: str
    """

    def __init__(self, endpoint: str = "https://127.0.0.1", **kwargs: Any) -> None:

        self.endpoint = endpoint
        kwargs.setdefault("sdk_moniker", "openmeter/{}".format(VERSION))
        self.polling_interval = kwargs.get("polling_interval", 30)
        self._configure(**kwargs)

    def _configure(self, **kwargs: Any) -> None:
        self.user_agent_policy = kwargs.get("user_agent_policy") or policies.UserAgentPolicy(**kwargs)
        self.headers_policy = kwargs.get("headers_policy") or policies.HeadersPolicy(**kwargs)
        self.proxy_policy = kwargs.get("proxy_policy") or policies.ProxyPolicy(**kwargs)
        self.logging_policy = kwargs.get("logging_policy") or policies.NetworkTraceLoggingPolicy(**kwargs)
        self.retry_policy = kwargs.get("retry_policy") or policies.RetryPolicy(**kwargs)
        self.authentication_policy = kwargs.get("authentication_policy")



# coding=utf-8

from enum import Enum
from corehttp.utils import CaseInsensitiveEnumMeta


class AddonInstanceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The instanceType of the add-on. Single instance add-ons can be added to subscription only once
    while add-ons with multiple type can be added more then once.
    """

    SINGLE = "single"
    """SINGLE."""
    MULTIPLE = "multiple"
    """MULTIPLE."""


class AddonOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for add-ons."""

    ID = "id"
    """ID."""
    KEY = "key"
    """KEY."""
    VERSION = "version"
    """VERSION."""
    CREATED_AT = "created_at"
    """CREATED_AT."""
    UPDATED_AT = "updated_at"
    """UPDATED_AT."""


class AddonStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The status of the add-on defined by the effectiveFrom and effectiveTo properties."""

    DRAFT = "draft"
    """DRAFT."""
    ACTIVE = "active"
    """ACTIVE."""
    ARCHIVED = "archived"
    """ARCHIVED."""


class AppCapabilityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """App capability type."""

    REPORT_USAGE = "reportUsage"
    """The app can report aggregated usage."""
    REPORT_EVENTS = "reportEvents"
    """The app can report raw events."""
    CALCULATE_TAX = "calculateTax"
    """The app can calculate tax."""
    INVOICE_CUSTOMERS = "invoiceCustomers"
    """The app can invoice customers."""
    COLLECT_PAYMENTS = "collectPayments"
    """The app can collect payments."""


class AppStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """App installed status."""

    READY = "ready"
    """The app is ready to be used."""
    UNAUTHORIZED = "unauthorized"
    """The app is unauthorized. This usually happens when the app's credentials are revoked or
    expired. To resolve this, the user must re-authorize the app."""


class AppType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the app."""

    STRIPE = "stripe"
    """STRIPE."""
    SANDBOX = "sandbox"
    """SANDBOX."""
    CUSTOM_INVOICING = "custom_invoicing"
    """CUSTOM_INVOICING."""


class BillingCollectionAlignment(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Collection alignment."""

    SUBSCRIPTION = "subscription"
    """Align the collection to the start of the subscription period."""
    ANCHORED = "anchored"
    """Align the collection to the anchor time and cadence."""


class BillingProfileCustomerOverrideExpand(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """CustomerOverrideExpand specifies the parts of the profile to expand."""

    APPS = "apps"
    """APPS."""
    CUSTOMER = "customer"
    """CUSTOMER."""


class BillingProfileCustomerOverrideOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for customers."""

    CUSTOMER_ID = "customerId"
    """CUSTOMER_ID."""
    CUSTOMER_NAME = "customerName"
    """CUSTOMER_NAME."""
    CUSTOMER_KEY = "customerKey"
    """CUSTOMER_KEY."""
    CUSTOMER_PRIMARY_EMAIL = "customerPrimaryEmail"
    """CUSTOMER_PRIMARY_EMAIL."""
    CUSTOMER_CREATED_AT = "customerCreatedAt"
    """CUSTOMER_CREATED_AT."""


class BillingProfileExpand(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BillingProfileExpand details what profile fields to expand."""

    APPS = "apps"
    """APPS."""


class BillingProfileOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """BillingProfileOrderBy specifies the ordering options for profiles."""

    CREATED_AT = "createdAt"
    """CREATED_AT."""
    UPDATED_AT = "updatedAt"
    """UPDATED_AT."""
    DEFAULT = "default"
    """DEFAULT."""
    NAME = "name"
    """NAME."""


class CheckoutSessionUIMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Stripe CheckoutSession.ui_mode."""

    EMBEDDED = "embedded"
    """EMBEDDED."""
    HOSTED = "hosted"
    """HOSTED."""


class CollectionMethod(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Collection method."""

    CHARGE_AUTOMATICALLY = "charge_automatically"
    """CHARGE_AUTOMATICALLY."""
    SEND_INVOICE = "send_invoice"
    """SEND_INVOICE."""


class CreateCheckoutSessionTaxIdCollectionRequired(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """Create Stripe checkout session tax ID collection required."""

    IF_SUPPORTED = "if_supported"
    """A tax ID will be required if collection is supported for the selected billing address country.
    See: `https://docs.stripe.com/tax/checkout/tax-ids#supported-types
    <https://docs.stripe.com/tax/checkout/tax-ids#supported-types>`_."""
    NEVER = "never"
    """Tax ID collection is never required."""


class CreateStripeCheckoutSessionBillingAddressCollection(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """Specify whether Checkout should collect the customer’s billing address."""

    AUTO = "auto"
    """Checkout will only collect the billing address when necessary. When using automatic_tax,
    Checkout will collect the minimum number of fields required for tax calculation."""
    REQUIRED = "required"
    """Checkout will always collect the customer’s billing address."""


class CreateStripeCheckoutSessionConsentCollectionPaymentMethodReuseAgreementPosition(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """Create Stripe checkout session consent collection agreement position."""

    AUTO = "auto"
    """Uses Stripe defaults to determine the visibility and position of the payment method reuse
    agreement."""
    HIDDEN = "hidden"
    """Hides the payment method reuse agreement."""


class CreateStripeCheckoutSessionConsentCollectionPromotions(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """Create Stripe checkout session consent collection promotions."""

    AUTO = "auto"
    """Enable the collection of customer consent for promotional communications. The Checkout Session
    will determine whether to display an option to opt into promotional communication from the
    merchant depending on if a customer is provided, and if that customer has consented to
    receiving promotional communications from the merchant in the past."""
    NONE = "none"
    """Checkout will not collect customer consent for promotional communications."""


class CreateStripeCheckoutSessionConsentCollectionTermsOfService(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """Create Stripe checkout session consent collection terms of service."""

    NONE = "none"
    """Does not display checkbox for the terms of service agreement."""
    REQUIRED = "required"
    """Displays a checkbox for the terms of service agreement which requires customer to check before
    being able to pay."""


class CreateStripeCheckoutSessionCustomerUpdateBehavior(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """Create Stripe checkout session customer update behavior."""

    AUTO = "auto"
    """Checkout will automatically determine whether to update the provided Customer object using
    details from the session."""
    NEVER = "never"
    """Checkout will never update the provided Customer object."""


class CreateStripeCheckoutSessionRedirectOnCompletion(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """Create Stripe checkout session redirect on completion."""

    ALWAYS = "always"
    """The Session will always redirect to the return_url after successful confirmation."""
    IF_REQUIRED = "if_required"
    """The Session will only redirect to the return_url after a redirect-based payment method is used."""
    NEVER = "never"
    """The Session will never redirect to the return_url, and redirect-based payment methods will be
    disabled."""


class CustomerExpand(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """CustomerExpand specifies the parts of the customer to expand in the list output."""

    SUBSCRIPTIONS = "subscriptions"
    """SUBSCRIPTIONS."""


class CustomerOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for customers."""

    ID = "id"
    """ID."""
    NAME = "name"
    """NAME."""
    CREATED_AT = "createdAt"
    """CREATED_AT."""


class CustomerSubscriptionOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for customer subscriptions."""

    ACTIVE_FROM = "activeFrom"
    """ACTIVE_FROM."""
    ACTIVE_TO = "activeTo"
    """ACTIVE_TO."""


class CustomInvoicingPaymentTrigger(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Payment trigger to execute on a finalized invoice."""

    PAID = "paid"
    """PAID."""
    PAYMENT_FAILED = "payment_failed"
    """PAYMENT_FAILED."""
    PAYMENT_UNCOLLECTIBLE = "payment_uncollectible"
    """PAYMENT_UNCOLLECTIBLE."""
    PAYMENT_OVERDUE = "payment_overdue"
    """PAYMENT_OVERDUE."""
    ACTION_REQUIRED = "action_required"
    """ACTION_REQUIRED."""
    VOID = "void"
    """VOID."""


class DiscountReasonType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the discount reason."""

    MAXIMUM_SPEND = "maximum_spend"
    """MAXIMUM_SPEND."""
    RATECARD_PERCENTAGE = "ratecard_percentage"
    """RATECARD_PERCENTAGE."""
    RATECARD_USAGE = "ratecard_usage"
    """RATECARD_USAGE."""


class EditOp(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum listing the different operation types."""

    ADD_ITEM = "add_item"
    """ADD_ITEM."""
    REMOVE_ITEM = "remove_item"
    """REMOVE_ITEM."""
    UNSCHEDULE_EDIT = "unschedule_edit"
    """UNSCHEDULE_EDIT."""
    ADD_PHASE = "add_phase"
    """ADD_PHASE."""
    REMOVE_PHASE = "remove_phase"
    """REMOVE_PHASE."""
    STRETCH_PHASE = "stretch_phase"
    """STRETCH_PHASE."""


class EntitlementOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for entitlements."""

    CREATED_AT = "createdAt"
    """CREATED_AT."""
    UPDATED_AT = "updatedAt"
    """UPDATED_AT."""


class EntitlementType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the entitlement."""

    METERED = "metered"
    """METERED."""
    BOOLEAN = "boolean"
    """BOOLEAN."""
    STATIC = "static"
    """STATIC."""


class ExpirationDuration(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The expiration duration enum."""

    HOUR = "HOUR"
    """HOUR."""
    DAY = "DAY"
    """DAY."""
    WEEK = "WEEK"
    """WEEK."""
    MONTH = "MONTH"
    """MONTH."""
    YEAR = "YEAR"
    """YEAR."""


class FeatureOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for features."""

    ID = "id"
    """ID."""
    KEY = "key"
    """KEY."""
    NAME = "name"
    """NAME."""
    CREATED_AT = "createdAt"
    """CREATED_AT."""
    UPDATED_AT = "updatedAt"
    """UPDATED_AT."""


class FeatureUnitCostType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of unit cost."""

    LLM = "llm"
    """LLM."""
    MANUAL = "manual"
    """MANUAL."""


class GrantOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for grants."""

    ID = "id"
    """ID."""
    CREATED_AT = "createdAt"
    """CREATED_AT."""
    UPDATED_AT = "updatedAt"
    """UPDATED_AT."""


class InstallMethod(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Install method of the application."""

    WITH_O_AUTH2 = "with_oauth2"
    """WITH_O_AUTH2."""
    WITH_API_KEY = "with_api_key"
    """WITH_API_KEY."""
    NO_CREDENTIALS_REQUIRED = "no_credentials_required"
    """NO_CREDENTIALS_REQUIRED."""


class InvoiceDetailedLineCostCategory(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """InvoiceDetailedLineCostCategory determines if the flat fee is a regular fee due to use due to a
    commitment.
    """

    REGULAR = "regular"
    """The fee is a regular fee due to usage."""
    COMMITMENT = "commitment"
    """The fee is a fee due to a commitment (e.g. minimum spend)."""


class InvoiceDocumentRefType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """InvoiceDocumentRefType defines the type of document that is being referenced."""

    CREDIT_NOTE_ORIGINAL_INVOICE = "credit_note_original_invoice"
    """CREDIT_NOTE_ORIGINAL_INVOICE."""


class InvoiceExpand(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """InvoiceExpand specifies the parts of the invoice to expand in the list output."""

    LINES = "lines"
    """LINES."""
    PRECEDING = "preceding"
    """PRECEDING."""
    WORKFLOW_APPS = "workflow.apps"
    """WORKFLOW_APPS."""


class InvoiceLineManagedBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """InvoiceLineManagedBy specifies who manages the line."""

    SUBSCRIPTION = "subscription"
    """The line is managed by the susbcription engine of
    
    If there are any changes to the subscription the line will be updated accordingly."""
    SYSTEM = "system"
    """The line is managed by the billing system of the
    
    The line is immutable."""
    MANUAL = "manual"
    """The line is managed via our API.
    
    If the line is coming from a subscription we will not update the line if the subscription
    changes.
    
    The only exception is that the period and invoiceAt fields will be updated in case of
    progressively billed
    usage-based lines to maintain the coherence of the line structure. Any other fields edited will
    be kept as is."""


class InvoiceLineStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Line status specifies the status of the line."""

    VALID = "valid"
    """The line is valid and can be used in the invoice."""
    DETAILED = "detailed"
    """The line is a detail line which is used to detail the individual charges and discounts of a
    valid line."""
    SPLIT = "split"
    """The line has been split into multiple valid lines due to progressive billing."""


class InvoiceLineTaxBehavior(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """InvoiceLineTaxBehavior details how the tax item is applied to the base amount.

    Inclusive means the tax is included in the base amount.
    Exclusive means the tax is added to the base amount.
    """

    INCLUSIVE = "inclusive"
    """Tax is included in the base amount."""
    EXCLUSIVE = "exclusive"
    """Tax is added to the base amount."""


class InvoiceLineTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """LineTypes represents the different types of lines that can be used in an invoice."""

    FLAT_FEE = "flat_fee"
    """FLAT_FEE."""
    USAGE_BASED = "usage_based"
    """USAGE_BASED."""


class InvoiceOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """InvoiceOrderBy specifies the ordering options for invoice listing."""

    CUSTOMER_NAME = "customer.name"
    """CUSTOMER_NAME."""
    ISSUED_AT = "issuedAt"
    """ISSUED_AT."""
    STATUS = "status"
    """STATUS."""
    CREATED_AT = "createdAt"
    """CREATED_AT."""
    UPDATED_AT = "updatedAt"
    """UPDATED_AT."""
    PERIOD_START = "periodStart"
    """PERIOD_START."""


class InvoiceStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """InvoiceStatus describes the status of an invoice."""

    GATHERING = "gathering"
    """The list of line items for the next invoice is being gathered."""
    DRAFT = "draft"
    """The invoice is in draft status."""
    ISSUING = "issuing"
    """The invoice is in the process of being issued."""
    ISSUED = "issued"
    """The invoice has been issued to the customer."""
    PAYMENT_PROCESSING = "payment_processing"
    """The payment for the invoice is being processed."""
    OVERDUE = "overdue"
    """The invoice's payment is overdue."""
    PAID = "paid"
    """The invoice has been paid."""
    UNCOLLECTIBLE = "uncollectible"
    """The invoice has been marked uncollectible."""
    VOIDED = "voided"
    """The invoice has been voided."""


class InvoiceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """InvoiceType represents the type of invoice.

    The type of invoice determines the purpose of the invoice and how it should be handled.
    """

    STANDARD = "standard"
    """A regular commercial invoice document between a supplier and customer."""
    CREDIT_NOTE = "credit_note"
    """Reflects a refund either partial or complete of the preceding document. A credit note
    effectively *extends* the previous document."""


class MeasureUsageFromPreset(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Start of measurement options."""

    CURRENT_PERIOD_START = "CURRENT_PERIOD_START"
    """CURRENT_PERIOD_START."""
    NOW = "NOW"
    """NOW."""


class MeterAggregation(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The aggregation type to use for the meter."""

    SUM = "SUM"
    """SUM."""
    COUNT = "COUNT"
    """COUNT."""
    UNIQUE_COUNT = "UNIQUE_COUNT"
    """UNIQUE_COUNT."""
    AVG = "AVG"
    """AVG."""
    MIN = "MIN"
    """MIN."""
    MAX = "MAX"
    """MAX."""
    LATEST = "LATEST"
    """LATEST."""


class MeterOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for meters."""

    KEY = "key"
    """KEY."""
    NAME = "name"
    """NAME."""
    AGGREGATION = "aggregation"
    """AGGREGATION."""
    CREATED_AT = "createdAt"
    """CREATED_AT."""
    UPDATED_AT = "updatedAt"
    """UPDATED_AT."""


class NotificationChannelOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for notification channels."""

    ID = "id"
    """ID."""
    TYPE = "type"
    """TYPE."""
    CREATED_AT = "createdAt"
    """CREATED_AT."""
    UPDATED_AT = "updatedAt"
    """UPDATED_AT."""


class NotificationChannelType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the notification channel."""

    WEBHOOK = "WEBHOOK"
    """WEBHOOK."""


class NotificationEventDeliveryStatusState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Delivery State."""

    SUCCESS = "SUCCESS"
    """SUCCESS."""
    FAILED = "FAILED"
    """FAILED."""
    SENDING = "SENDING"
    """SENDING."""
    PENDING = "PENDING"
    """PENDING."""
    RESENDING = "RESENDING"
    """RESENDING."""


class NotificationEventOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for notification channels."""

    ID = "id"
    """ID."""
    CREATED_AT = "createdAt"
    """CREATED_AT."""


class NotificationEventType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the notification event."""

    ENTITLEMENTS_BALANCE_THRESHOLD = "entitlements.balance.threshold"
    """ENTITLEMENTS_BALANCE_THRESHOLD."""
    ENTITLEMENTS_RESET = "entitlements.reset"
    """ENTITLEMENTS_RESET."""
    INVOICE_CREATED = "invoice.created"
    """INVOICE_CREATED."""
    INVOICE_UPDATED = "invoice.updated"
    """INVOICE_UPDATED."""


class NotificationRuleBalanceThresholdValueType(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """Notification balance threshold type."""

    PERCENT = "PERCENT"
    """PERCENT."""
    NUMBER = "NUMBER"
    """NUMBER."""
    BALANCE_VALUE = "balance_value"
    """BALANCE_VALUE."""
    USAGE_PERCENTAGE = "usage_percentage"
    """USAGE_PERCENTAGE."""
    USAGE_VALUE = "usage_value"
    """USAGE_VALUE."""


class NotificationRuleOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for notification channels."""

    ID = "id"
    """ID."""
    TYPE = "type"
    """TYPE."""
    CREATED_AT = "createdAt"
    """CREATED_AT."""
    UPDATED_AT = "updatedAt"
    """UPDATED_AT."""


class OAuth2AuthorizationCodeGrantErrorType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """OAuth2 authorization code grant error types."""

    INVALID_REQUEST = "invalid_request"
    """The request is missing a required parameter, includes an invalid parameter value, includes a
    parameter more than once, or is otherwise malformed."""
    UNAUTHORIZED_CLIENT = "unauthorized_client"
    """The client is not authorized to request an authorization code using this method."""
    ACCESS_DENIED = "access_denied"
    """The resource owner or authorization server denied the request."""
    UNSUPPORTED_RESPONSE_TYPE = "unsupported_response_type"
    """The authorization server does not support obtaining an authorization code using this method."""
    INVALID_SCOPE = "invalid_scope"
    """The requested scope is invalid, unknown, or malformed."""
    SERVER_ERROR = "server_error"
    """The authorization server encountered an unexpected condition that prevented it from fulfilling
    the request."""
    TEMPORARILY_UNAVAILABLE = "temporarily_unavailable"
    """The authorization server is currently unable to handle the request due to a temporary
    overloading or maintenance of the server."""


class PaymentTermType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """PaymentTermType defines the type of terms to be applied."""

    DUE_DATE = "due_date"
    """Due on a specific date."""
    INSTANT = "instant"
    """On receipt of invoice."""


class PlanAddonOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for plan add-on assignments."""

    ID = "id"
    """ID."""
    KEY = "key"
    """KEY."""
    VERSION = "version"
    """VERSION."""
    CREATED_AT = "created_at"
    """CREATED_AT."""
    UPDATED_AT = "updated_at"
    """UPDATED_AT."""


class PlanOrderBy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Order by options for plans."""

    ID = "id"
    """ID."""
    KEY = "key"
    """KEY."""
    VERSION = "version"
    """VERSION."""
    CREATED_AT = "created_at"
    """CREATED_AT."""
    UPDATED_AT = "updated_at"
    """UPDATED_AT."""


class PlanStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The status of a plan."""

    DRAFT = "draft"
    """DRAFT."""
    ACTIVE = "active"
    """ACTIVE."""
    ARCHIVED = "archived"
    """ARCHIVED."""
    SCHEDULED = "scheduled"
    """SCHEDULED."""


class PricePaymentTerm(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The payment term of a flat price. One of: in_advance or in_arrears."""

    IN_ADVANCE = "in_advance"
    """If in_advance, the rate card will be invoiced in the previous billing cycle."""
    IN_ARREARS = "in_arrears"
    """If in_arrears, the rate card will be invoiced in the current billing cycle."""


class PriceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the price."""

    FLAT = "flat"
    """FLAT."""
    UNIT = "unit"
    """UNIT."""
    TIERED = "tiered"
    """TIERED."""
    DYNAMIC = "dynamic"
    """DYNAMIC."""
    PACKAGE = "package"
    """PACKAGE."""


class ProRatingMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Pro-rating mode options for handling billing period changes."""

    PRORATE_PRICES = "prorate_prices"
    """Calculate pro-rated charges based on time remaining in billing period."""


class RateCardType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the rate card."""

    FLAT_FEE = "flat_fee"
    """FLAT_FEE."""
    USAGE_BASED = "usage_based"
    """USAGE_BASED."""


class RecurringPeriodIntervalEnum(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The unit of time for the interval. One of: ``day``, ``week``, ``month``, or ``year``."""

    DAY = "DAY"
    """DAY."""
    WEEK = "WEEK"
    """WEEK."""
    MONTH = "MONTH"
    """MONTH."""
    YEAR = "YEAR"
    """YEAR."""


class RemovePhaseShifting(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The direction of the phase shift when a phase is removed."""

    NEXT = "next"
    """Shifts all subsequent phases to start sooner by the deleted phase's length."""
    PREV = "prev"
    """Extends the previous phase to end later by the deleted phase's length."""


class SortOrder(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The order direction."""

    ASC = "ASC"
    """ASC."""
    DESC = "DESC"
    """DESC."""


class StripeCheckoutSessionMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Stripe CheckoutSession.mode."""

    SETUP = "setup"
    """SETUP."""


class SubscriptionStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Subscription status."""

    ACTIVE = "active"
    """ACTIVE."""
    INACTIVE = "inactive"
    """INACTIVE."""
    CANCELED = "canceled"
    """CANCELED."""
    SCHEDULED = "scheduled"
    """SCHEDULED."""


class SubscriptionTimingEnum(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Subscription edit timing. When immediate, the requested changes take effect immediately. When
    nextBillingCycle, the requested changes take effect at the next billing cycle.
    """

    IMMEDIATE = "immediate"
    """IMMEDIATE."""
    NEXT_BILLING_CYCLE = "next_billing_cycle"
    """NEXT_BILLING_CYCLE."""


class TaxBehavior(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Tax behavior.

    This enum is used to specify whether tax is included in the price or excluded from the price.
    """

    INCLUSIVE = "inclusive"
    """Tax is included in the price."""
    EXCLUSIVE = "exclusive"
    """Tax is excluded from the price."""


class TieredPriceMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The mode of the tiered price."""

    VOLUME = "volume"
    """VOLUME."""
    GRADUATED = "graduated"
    """GRADUATED."""


class ValidationIssueSeverity(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ValidationIssueSeverity describes the severity of a validation issue.

    Issues with severity "critical" will prevent the invoice from being issued.
    """

    CRITICAL = "critical"
    """CRITICAL."""
    WARNING = "warning"
    """WARNING."""


class VoidInvoiceLineActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """VoidInvoiceLineActionType describes how to handle the voidied line item in the invoice."""

    DISCARD = "discard"
    """The line items will never be charged for again."""
    PENDING = "pending"
    """Queue the line items into the pending state, they will be included in the next invoice. (We
    want to generate an invoice right now)."""


class WindowSize(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Aggregation window size."""

    MINUTE = "MINUTE"
    """MINUTE."""
    HOUR = "HOUR"
    """HOUR."""
    DAY = "DAY"
    """DAY."""
    MONTH = "MONTH"
    """MONTH."""



# coding=utf-8
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    Addon,
    AddonCreate,
    AddonReplaceUpdate,
    Address,
    Alignment,
    Annotations,
    AppCapability,
    AppPaginatedResponse,
    AppReference,
    BadRequestProblemResponse,
    BalanceHistoryWindow,
    BillingCustomerProfile,
    BillingDiscountPercentage,
    BillingDiscountUsage,
    BillingDiscounts,
    BillingInvoiceCustomerExtendedDetails,
    BillingParty,
    BillingPartyReplaceUpdate,
    BillingPartyTaxIdentity,
    BillingProfile,
    BillingProfileAppReferences,
    BillingProfileApps,
    BillingProfileAppsCreate,
    BillingProfileCreate,
    BillingProfileCustomerOverride,
    BillingProfileCustomerOverrideCreate,
    BillingProfileCustomerOverrideWithDetails,
    BillingProfileCustomerOverrideWithDetailsPaginatedResponse,
    BillingProfilePaginatedResponse,
    BillingProfileReplaceUpdateWithWorkflow,
    BillingWorkflow,
    BillingWorkflowCollectionAlignmentAnchored,
    BillingWorkflowCollectionAlignmentSubscription,
    BillingWorkflowCollectionSettings,
    BillingWorkflowCreate,
    BillingWorkflowInvoicingSettings,
    BillingWorkflowPaymentSettings,
    BillingWorkflowTaxSettings,
    CancelRequest,
    CheckoutSessionCustomTextAfterSubmitParams,
    CheckoutSessionCustomTextParamsAfterSubmit,
    CheckoutSessionCustomTextParamsShippingAddress,
    CheckoutSessionCustomTextParamsSubmit,
    CheckoutSessionCustomTextParamsTermsOfServiceAcceptance,
    ClientAppStartResponse,
    ConflictProblemResponse,
    CreateCheckoutSessionTaxIdCollection,
    CreateResponseExtensions,
    CreateStripeCheckoutSessionConsentCollection,
    CreateStripeCheckoutSessionConsentCollectionPaymentMethodReuseAgreement,
    CreateStripeCheckoutSessionCustomerUpdate,
    CreateStripeCheckoutSessionRequest,
    CreateStripeCheckoutSessionRequestOptions,
    CreateStripeCheckoutSessionResult,
    CreateStripeCustomerPortalSessionParams,
    CreditNoteOriginalInvoiceRef,
    Currency,
    CustomInvoicingApp,
    CustomInvoicingAppReplaceUpdate,
    CustomInvoicingCustomerAppData,
    CustomInvoicingDraftSynchronizedRequest,
    CustomInvoicingFinalizedInvoicingRequest,
    CustomInvoicingFinalizedPaymentRequest,
    CustomInvoicingFinalizedRequest,
    CustomInvoicingLineDiscountExternalIdMapping,
    CustomInvoicingLineExternalIdMapping,
    CustomInvoicingSyncResult,
    CustomInvoicingTaxConfig,
    CustomInvoicingUpdatePaymentStatusRequest,
    CustomPlanInput,
    CustomSubscriptionChange,
    CustomSubscriptionCreate,
    Customer,
    CustomerAccess,
    CustomerAppDataPaginatedResponse,
    CustomerCreate,
    CustomerId,
    CustomerKey,
    CustomerPaginatedResponse,
    CustomerReplaceUpdate,
    CustomerUsageAttribution,
    DiscountPercentage,
    DiscountReasonMaximumSpend,
    DiscountReasonRatecardPercentage,
    DiscountReasonRatecardUsage,
    DiscountUsage,
    Discounts,
    DynamicPriceWithCommitments,
    EditSubscriptionAddItem,
    EditSubscriptionAddPhase,
    EditSubscriptionRemoveItem,
    EditSubscriptionRemovePhase,
    EditSubscriptionStretchPhase,
    EditSubscriptionUnscheduleEdit,
    EntitlementBoolean,
    EntitlementBooleanCreateInputs,
    EntitlementBooleanV2,
    EntitlementGrant,
    EntitlementGrantCreateInput,
    EntitlementGrantCreateInputV2,
    EntitlementGrantV2,
    EntitlementMetered,
    EntitlementMeteredCreateInputs,
    EntitlementMeteredV2,
    EntitlementMeteredV2CreateInputs,
    EntitlementPaginatedResponse,
    EntitlementStatic,
    EntitlementStaticCreateInputs,
    EntitlementStaticV2,
    EntitlementV2PaginatedResponse,
    EntitlementValue,
    EntitlementValueV2,
    ErrorExtension,
    Event,
    EventDeliveryAttemptResponse,
    ExpirationPeriod,
    Feature,
    FeatureCreateInputs,
    FeatureLLMUnitCost,
    FeatureLLMUnitCostPricing,
    FeatureManualUnitCost,
    FeatureMeta,
    FeaturePaginatedResponse,
    FilterIDExact,
    FilterString,
    FilterTime,
    FlatPrice,
    FlatPriceWithPaymentTerm,
    ForbiddenProblemResponse,
    GrantBurnDownHistorySegment,
    GrantPaginatedResponse,
    GrantUsageRecord,
    GrantV2PaginatedResponse,
    IDResource,
    IngestedEvent,
    InstallWithApiKeyRequest,
    InternalServerErrorProblemResponse,
    Invoice,
    InvoiceAppExternalIds,
    InvoiceAvailableActionDetails,
    InvoiceAvailableActionInvoiceDetails,
    InvoiceAvailableActions,
    InvoiceDetailedLine,
    InvoiceDetailedLineRateCard,
    InvoiceGenericDocumentRef,
    InvoiceLine,
    InvoiceLineAmountDiscount,
    InvoiceLineAppExternalIds,
    InvoiceLineDiscounts,
    InvoiceLineReplaceUpdate,
    InvoiceLineSubscriptionReference,
    InvoiceLineTaxItem,
    InvoiceLineUsageDiscount,
    InvoicePaginatedResponse,
    InvoicePaymentTerms,
    InvoicePendingLineCreate,
    InvoicePendingLineCreateInput,
    InvoicePendingLineCreateResponse,
    InvoicePendingLinesActionFiltersInput,
    InvoicePendingLinesActionInput,
    InvoiceReference,
    InvoiceReplaceUpdate,
    InvoiceSimulationInput,
    InvoiceSimulationLine,
    InvoiceStatusDetails,
    InvoiceTotals,
    InvoiceUsageBasedRateCard,
    InvoiceWorkflowInvoicingSettingsReplaceUpdate,
    InvoiceWorkflowReplaceUpdate,
    InvoiceWorkflowSettings,
    InvoiceWorkflowSettingsReplaceUpdate,
    IssueAfterReset,
    ListRequestFilter,
    MarketplaceInstallRequestPayload,
    MarketplaceInstallResponse,
    MarketplaceListing,
    MarketplaceListingPaginatedResponse,
    Metadata,
    Meter,
    MeterCreate,
    MeterQueryRequest,
    MeterQueryResult,
    MeterQueryRow,
    MeterUpdate,
    MigrateRequest,
    NotFoundProblemResponse,
    NotificationChannelMeta,
    NotificationChannelPaginatedResponse,
    NotificationChannelWebhook,
    NotificationChannelWebhookCreateRequest,
    NotificationEvent,
    NotificationEventBalanceThresholdPayload,
    NotificationEventBalanceThresholdPayloadData,
    NotificationEventDeliveryAttempt,
    NotificationEventDeliveryStatus,
    NotificationEventEntitlementValuePayloadBase,
    NotificationEventInvoiceCreatedPayload,
    NotificationEventInvoiceUpdatedPayload,
    NotificationEventPaginatedResponse,
    NotificationEventResendRequest,
    NotificationEventResetPayload,
    NotificationRuleBalanceThreshold,
    NotificationRuleBalanceThresholdCreateRequest,
    NotificationRuleBalanceThresholdValue,
    NotificationRuleEntitlementReset,
    NotificationRuleEntitlementResetCreateRequest,
    NotificationRuleInvoiceCreated,
    NotificationRuleInvoiceCreatedCreateRequest,
    NotificationRuleInvoiceUpdated,
    NotificationRuleInvoiceUpdatedCreateRequest,
    NotificationRulePaginatedResponse,
    OmitPropertiesResourceCreateModel,
    PackagePriceWithCommitments,
    PaymentDueDate,
    PaymentTermDueDate,
    PaymentTermInstant,
    Period,
    Plan,
    PlanAddon,
    PlanAddonCreate,
    PlanAddonPaginatedResponse,
    PlanAddonReplaceUpdate,
    PlanCreate,
    PlanPhase,
    PlanReference,
    PlanReferenceInput,
    PlanReplaceUpdate,
    PlanSubscriptionChange,
    PlanSubscriptionCreate,
    PortalToken,
    PreconditionFailedProblemResponse,
    PriceTier,
    ProRatingConfig,
    Progress,
    RateCardBooleanEntitlement,
    RateCardFlatFee,
    RateCardMeteredEntitlement,
    RateCardStaticEntitlement,
    RateCardUsageBased,
    RecurringPeriod,
    RecurringPeriodCreateInput,
    RecurringPeriodV2,
    ResetEntitlementUsageInput,
    SandboxApp,
    SandboxAppReplaceUpdate,
    SandboxCustomerAppData,
    ServiceUnavailableProblemResponse,
    StripeAPIKeyInput,
    StripeApp,
    StripeAppReplaceUpdate,
    StripeCustomerAppData,
    StripeCustomerAppDataBase,
    StripeCustomerPortalSession,
    StripeTaxConfig,
    StripeWebhookEvent,
    StripeWebhookEventData,
    StripeWebhookResponse,
    Subject,
    SubjectUpsert,
    Subscription,
    SubscriptionAddon,
    SubscriptionAddonAddon,
    SubscriptionAddonCreate,
    SubscriptionAddonCreateAddon,
    SubscriptionAddonRateCard,
    SubscriptionAddonTimelineSegment,
    SubscriptionAddonUpdate,
    SubscriptionAlignment,
    SubscriptionBadRequestErrorResponse,
    SubscriptionChangeResponseBody,
    SubscriptionConflictErrorResponse,
    SubscriptionEdit,
    SubscriptionExpanded,
    SubscriptionItem,
    SubscriptionItemIncluded,
    SubscriptionPaginatedResponse,
    SubscriptionPhaseCreate,
    SubscriptionPhaseExpanded,
    TaxConfig,
    TieredPriceWithCommitments,
    UnauthorizedProblemResponse,
    UnexpectedProblemResponse,
    UnitPrice,
    UnitPriceWithCommitments,
    ValidationError,
    ValidationIssue,
    VoidInvoiceAction,
    VoidInvoiceActionInput,
    VoidInvoiceActionLineOverride,
    VoidInvoiceLineDiscardAction,
    VoidInvoiceLinePendingAction,
    WindowedBalanceHistory,
)

from ._enums import (  # type: ignore
    AddonInstanceType,
    AddonOrderBy,
    AddonStatus,
    AppCapabilityType,
    AppStatus,
    AppType,
    BillingCollectionAlignment,
    BillingProfileCustomerOverrideExpand,
    BillingProfileCustomerOverrideOrderBy,
    BillingProfileExpand,
    BillingProfileOrderBy,
    CheckoutSessionUIMode,
    CollectionMethod,
    CreateCheckoutSessionTaxIdCollectionRequired,
    CreateStripeCheckoutSessionBillingAddressCollection,
    CreateStripeCheckoutSessionConsentCollectionPaymentMethodReuseAgreementPosition,
    CreateStripeCheckoutSessionConsentCollectionPromotions,
    CreateStripeCheckoutSessionConsentCollectionTermsOfService,
    CreateStripeCheckoutSessionCustomerUpdateBehavior,
    CreateStripeCheckoutSessionRedirectOnCompletion,
    CustomInvoicingPaymentTrigger,
    CustomerExpand,
    CustomerOrderBy,
    CustomerSubscriptionOrderBy,
    DiscountReasonType,
    EditOp,
    EntitlementOrderBy,
    EntitlementType,
    ExpirationDuration,
    FeatureOrderBy,
    FeatureUnitCostType,
    GrantOrderBy,
    InstallMethod,
    InvoiceDetailedLineCostCategory,
    InvoiceDocumentRefType,
    InvoiceExpand,
    InvoiceLineManagedBy,
    InvoiceLineStatus,
    InvoiceLineTaxBehavior,
    InvoiceLineTypes,
    InvoiceOrderBy,
    InvoiceStatus,
    InvoiceType,
    MeasureUsageFromPreset,
    MeterAggregation,
    MeterOrderBy,
    NotificationChannelOrderBy,
    NotificationChannelType,
    NotificationEventDeliveryStatusState,
    NotificationEventOrderBy,
    NotificationEventType,
    NotificationRuleBalanceThresholdValueType,
    NotificationRuleOrderBy,
    OAuth2AuthorizationCodeGrantErrorType,
    PaymentTermType,
    PlanAddonOrderBy,
    PlanOrderBy,
    PlanStatus,
    PricePaymentTerm,
    PriceType,
    ProRatingMode,
    RateCardType,
    RecurringPeriodIntervalEnum,
    RemovePhaseShifting,
    SortOrder,
    StripeCheckoutSessionMode,
    SubscriptionStatus,
    SubscriptionTimingEnum,
    TaxBehavior,
    TieredPriceMode,
    ValidationIssueSeverity,
    VoidInvoiceLineActionType,
    WindowSize,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Addon",
    "AddonCreate",
    "AddonReplaceUpdate",
    "Address",
    "Alignment",
    "Annotations",
    "AppCapability",
    "AppPaginatedResponse",
    "AppReference",
    "BadRequestProblemResponse",
    "BalanceHistoryWindow",
    "BillingCustomerProfile",
    "BillingDiscountPercentage",
    "BillingDiscountUsage",
    "BillingDiscounts",
    "BillingInvoiceCustomerExtendedDetails",
    "BillingParty",
    "BillingPartyReplaceUpdate",
    "BillingPartyTaxIdentity",
    "BillingProfile",
    "BillingProfileAppReferences",
    "BillingProfileApps",
    "BillingProfileAppsCreate",
    "BillingProfileCreate",
    "BillingProfileCustomerOverride",
    "BillingProfileCustomerOverrideCreate",
    "BillingProfileCustomerOverrideWithDetails",
    "BillingProfileCustomerOverrideWithDetailsPaginatedResponse",
    "BillingProfilePaginatedResponse",
    "BillingProfileReplaceUpdateWithWorkflow",
    "BillingWorkflow",
    "BillingWorkflowCollectionAlignmentAnchored",
    "BillingWorkflowCollectionAlignmentSubscription",
    "BillingWorkflowCollectionSettings",
    "BillingWorkflowCreate",
    "BillingWorkflowInvoicingSettings",
    "BillingWorkflowPaymentSettings",
    "BillingWorkflowTaxSettings",
    "CancelRequest",
    "CheckoutSessionCustomTextAfterSubmitParams",
    "CheckoutSessionCustomTextParamsAfterSubmit",
    "CheckoutSessionCustomTextParamsShippingAddress",
    "CheckoutSessionCustomTextParamsSubmit",
    "CheckoutSessionCustomTextParamsTermsOfServiceAcceptance",
    "ClientAppStartResponse",
    "ConflictProblemResponse",
    "CreateCheckoutSessionTaxIdCollection",
    "CreateResponseExtensions",
    "CreateStripeCheckoutSessionConsentCollection",
    "CreateStripeCheckoutSessionConsentCollectionPaymentMethodReuseAgreement",
    "CreateStripeCheckoutSessionCustomerUpdate",
    "CreateStripeCheckoutSessionRequest",
    "CreateStripeCheckoutSessionRequestOptions",
    "CreateStripeCheckoutSessionResult",
    "CreateStripeCustomerPortalSessionParams",
    "CreditNoteOriginalInvoiceRef",
    "Currency",
    "CustomInvoicingApp",
    "CustomInvoicingAppReplaceUpdate",
    "CustomInvoicingCustomerAppData",
    "CustomInvoicingDraftSynchronizedRequest",
    "CustomInvoicingFinalizedInvoicingRequest",
    "CustomInvoicingFinalizedPaymentRequest",
    "CustomInvoicingFinalizedRequest",
    "CustomInvoicingLineDiscountExternalIdMapping",
    "CustomInvoicingLineExternalIdMapping",
    "CustomInvoicingSyncResult",
    "CustomInvoicingTaxConfig",
    "CustomInvoicingUpdatePaymentStatusRequest",
    "CustomPlanInput",
    "CustomSubscriptionChange",
    "CustomSubscriptionCreate",
    "Customer",
    "CustomerAccess",
    "CustomerAppDataPaginatedResponse",
    "CustomerCreate",
    "CustomerId",
    "CustomerKey",
    "CustomerPaginatedResponse",
    "CustomerReplaceUpdate",
    "CustomerUsageAttribution",
    "DiscountPercentage",
    "DiscountReasonMaximumSpend",
    "DiscountReasonRatecardPercentage",
    "DiscountReasonRatecardUsage",
    "DiscountUsage",
    "Discounts",
    "DynamicPriceWithCommitments",
    "EditSubscriptionAddItem",
    "EditSubscriptionAddPhase",
    "EditSubscriptionRemoveItem",
    "EditSubscriptionRemovePhase",
    "EditSubscriptionStretchPhase",
    "EditSubscriptionUnscheduleEdit",
    "EntitlementBoolean",
    "EntitlementBooleanCreateInputs",
    "EntitlementBooleanV2",
    "EntitlementGrant",
    "EntitlementGrantCreateInput",
    "EntitlementGrantCreateInputV2",
    "EntitlementGrantV2",
    "EntitlementMetered",
    "EntitlementMeteredCreateInputs",
    "EntitlementMeteredV2",
    "EntitlementMeteredV2CreateInputs",
    "EntitlementPaginatedResponse",
    "EntitlementStatic",
    "EntitlementStaticCreateInputs",
    "EntitlementStaticV2",
    "EntitlementV2PaginatedResponse",
    "EntitlementValue",
    "EntitlementValueV2",
    "ErrorExtension",
    "Event",
    "EventDeliveryAttemptResponse",
    "ExpirationPeriod",
    "Feature",
    "FeatureCreateInputs",
    "FeatureLLMUnitCost",
    "FeatureLLMUnitCostPricing",
    "FeatureManualUnitCost",
    "FeatureMeta",
    "FeaturePaginatedResponse",
    "FilterIDExact",
    "FilterString",
    "FilterTime",
    "FlatPrice",
    "FlatPriceWithPaymentTerm",
    "ForbiddenProblemResponse",
    "GrantBurnDownHistorySegment",
    "GrantPaginatedResponse",
    "GrantUsageRecord",
    "GrantV2PaginatedResponse",
    "IDResource",
    "IngestedEvent",
    "InstallWithApiKeyRequest",
    "InternalServerErrorProblemResponse",
    "Invoice",
    "InvoiceAppExternalIds",
    "InvoiceAvailableActionDetails",
    "InvoiceAvailableActionInvoiceDetails",
    "InvoiceAvailableActions",
    "InvoiceDetailedLine",
    "InvoiceDetailedLineRateCard",
    "InvoiceGenericDocumentRef",
    "InvoiceLine",
    "InvoiceLineAmountDiscount",
    "InvoiceLineAppExternalIds",
    "InvoiceLineDiscounts",
    "InvoiceLineReplaceUpdate",
    "InvoiceLineSubscriptionReference",
    "InvoiceLineTaxItem",
    "InvoiceLineUsageDiscount",
    "InvoicePaginatedResponse",
    "InvoicePaymentTerms",
    "InvoicePendingLineCreate",
    "InvoicePendingLineCreateInput",
    "InvoicePendingLineCreateResponse",
    "InvoicePendingLinesActionFiltersInput",
    "InvoicePendingLinesActionInput",
    "InvoiceReference",
    "InvoiceReplaceUpdate",
    "InvoiceSimulationInput",
    "InvoiceSimulationLine",
    "InvoiceStatusDetails",
    "InvoiceTotals",
    "InvoiceUsageBasedRateCard",
    "InvoiceWorkflowInvoicingSettingsReplaceUpdate",
    "InvoiceWorkflowReplaceUpdate",
    "InvoiceWorkflowSettings",
    "InvoiceWorkflowSettingsReplaceUpdate",
    "IssueAfterReset",
    "ListRequestFilter",
    "MarketplaceInstallRequestPayload",
    "MarketplaceInstallResponse",
    "MarketplaceListing",
    "MarketplaceListingPaginatedResponse",
    "Metadata",
    "Meter",
    "MeterCreate",
    "MeterQueryRequest",
    "MeterQueryResult",
    "MeterQueryRow",
    "MeterUpdate",
    "MigrateRequest",
    "NotFoundProblemResponse",
    "NotificationChannelMeta",
    "NotificationChannelPaginatedResponse",
    "NotificationChannelWebhook",
    "NotificationChannelWebhookCreateRequest",
    "NotificationEvent",
    "NotificationEventBalanceThresholdPayload",
    "NotificationEventBalanceThresholdPayloadData",
    "NotificationEventDeliveryAttempt",
    "NotificationEventDeliveryStatus",
    "NotificationEventEntitlementValuePayloadBase",
    "NotificationEventInvoiceCreatedPayload",
    "NotificationEventInvoiceUpdatedPayload",
    "NotificationEventPaginatedResponse",
    "NotificationEventResendRequest",
    "NotificationEventResetPayload",
    "NotificationRuleBalanceThreshold",
    "NotificationRuleBalanceThresholdCreateRequest",
    "NotificationRuleBalanceThresholdValue",
    "NotificationRuleEntitlementReset",
    "NotificationRuleEntitlementResetCreateRequest",
    "NotificationRuleInvoiceCreated",
    "NotificationRuleInvoiceCreatedCreateRequest",
    "NotificationRuleInvoiceUpdated",
    "NotificationRuleInvoiceUpdatedCreateRequest",
    "NotificationRulePaginatedResponse",
    "OmitPropertiesResourceCreateModel",
    "PackagePriceWithCommitments",
    "PaymentDueDate",
    "PaymentTermDueDate",
    "PaymentTermInstant",
    "Period",
    "Plan",
    "PlanAddon",
    "PlanAddonCreate",
    "PlanAddonPaginatedResponse",
    "PlanAddonReplaceUpdate",
    "PlanCreate",
    "PlanPhase",
    "PlanReference",
    "PlanReferenceInput",
    "PlanReplaceUpdate",
    "PlanSubscriptionChange",
    "PlanSubscriptionCreate",
    "PortalToken",
    "PreconditionFailedProblemResponse",
    "PriceTier",
    "ProRatingConfig",
    "Progress",
    "RateCardBooleanEntitlement",
    "RateCardFlatFee",
    "RateCardMeteredEntitlement",
    "RateCardStaticEntitlement",
    "RateCardUsageBased",
    "RecurringPeriod",
    "RecurringPeriodCreateInput",
    "RecurringPeriodV2",
    "ResetEntitlementUsageInput",
    "SandboxApp",
    "SandboxAppReplaceUpdate",
    "SandboxCustomerAppData",
    "ServiceUnavailableProblemResponse",
    "StripeAPIKeyInput",
    "StripeApp",
    "StripeAppReplaceUpdate",
    "StripeCustomerAppData",
    "StripeCustomerAppDataBase",
    "StripeCustomerPortalSession",
    "StripeTaxConfig",
    "StripeWebhookEvent",
    "StripeWebhookEventData",
    "StripeWebhookResponse",
    "Subject",
    "SubjectUpsert",
    "Subscription",
    "SubscriptionAddon",
    "SubscriptionAddonAddon",
    "SubscriptionAddonCreate",
    "SubscriptionAddonCreateAddon",
    "SubscriptionAddonRateCard",
    "SubscriptionAddonTimelineSegment",
    "SubscriptionAddonUpdate",
    "SubscriptionAlignment",
    "SubscriptionBadRequestErrorResponse",
    "SubscriptionChangeResponseBody",
    "SubscriptionConflictErrorResponse",
    "SubscriptionEdit",
    "SubscriptionExpanded",
    "SubscriptionItem",
    "SubscriptionItemIncluded",
    "SubscriptionPaginatedResponse",
    "SubscriptionPhaseCreate",
    "SubscriptionPhaseExpanded",
    "TaxConfig",
    "TieredPriceWithCommitments",
    "UnauthorizedProblemResponse",
    "UnexpectedProblemResponse",
    "UnitPrice",
    "UnitPriceWithCommitments",
    "ValidationError",
    "ValidationIssue",
    "VoidInvoiceAction",
    "VoidInvoiceActionInput",
    "VoidInvoiceActionLineOverride",
    "VoidInvoiceLineDiscardAction",
    "VoidInvoiceLinePendingAction",
    "WindowedBalanceHistory",
    "AddonInstanceType",
    "AddonOrderBy",
    "AddonStatus",
    "AppCapabilityType",
    "AppStatus",
    "AppType",
    "BillingCollectionAlignment",
    "BillingProfileCustomerOverrideExpand",
    "BillingProfileCustomerOverrideOrderBy",
    "BillingProfileExpand",
    "BillingProfileOrderBy",
    "CheckoutSessionUIMode",
    "CollectionMethod",
    "CreateCheckoutSessionTaxIdCollectionRequired",
    "CreateStripeCheckoutSessionBillingAddressCollection",
    "CreateStripeCheckoutSessionConsentCollectionPaymentMethodReuseAgreementPosition",
    "CreateStripeCheckoutSessionConsentCollectionPromotions",
    "CreateStripeCheckoutSessionConsentCollectionTermsOfService",
    "CreateStripeCheckoutSessionCustomerUpdateBehavior",
    "CreateStripeCheckoutSessionRedirectOnCompletion",
    "CustomInvoicingPaymentTrigger",
    "CustomerExpand",
    "CustomerOrderBy",
    "CustomerSubscriptionOrderBy",
    "DiscountReasonType",
    "EditOp",
    "EntitlementOrderBy",
    "EntitlementType",
    "ExpirationDuration",
    "FeatureOrderBy",
    "FeatureUnitCostType",
    "GrantOrderBy",
    "InstallMethod",
    "InvoiceDetailedLineCostCategory",
    "InvoiceDocumentRefType",
    "InvoiceExpand",
    "InvoiceLineManagedBy",
    "InvoiceLineStatus",
    "InvoiceLineTaxBehavior",
    "InvoiceLineTypes",
    "InvoiceOrderBy",
    "InvoiceStatus",
    "InvoiceType",
    "MeasureUsageFromPreset",
    "MeterAggregation",
    "MeterOrderBy",
    "NotificationChannelOrderBy",
    "NotificationChannelType",
    "NotificationEventDeliveryStatusState",
    "NotificationEventOrderBy",
    "NotificationEventType",
    "NotificationRuleBalanceThresholdValueType",
    "NotificationRuleOrderBy",
    "OAuth2AuthorizationCodeGrantErrorType",
    "PaymentTermType",
    "PlanAddonOrderBy",
    "PlanOrderBy",
    "PlanStatus",
    "PricePaymentTerm",
    "PriceType",
    "ProRatingMode",
    "RateCardType",
    "RecurringPeriodIntervalEnum",
    "RemovePhaseShifting",
    "SortOrder",
    "StripeCheckoutSessionMode",
    "SubscriptionStatus",
    "SubscriptionTimingEnum",
    "TaxBehavior",
    "TieredPriceMode",
    "ValidationIssueSeverity",
    "VoidInvoiceLineActionType",
    "WindowSize",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8

import datetime
from typing import TYPE_CHECKING, Union

if TYPE_CHECKING:
    from . import models as _models
App = Union["_models.StripeApp", "_models.SandboxApp", "_models.CustomInvoicingApp"]
CustomerAppData = Union[
    "_models.StripeCustomerAppData", "_models.SandboxCustomerAppData", "_models.CustomInvoicingCustomerAppData"
]
FeatureUnitCost = Union["_models.FeatureManualUnitCost", "_models.FeatureLLMUnitCost"]
RateCardEntitlement = Union[
    "_models.RateCardMeteredEntitlement", "_models.RateCardStaticEntitlement", "_models.RateCardBooleanEntitlement"
]
RateCardUsageBasedPrice = Union[
    "_models.FlatPriceWithPaymentTerm",
    "_models.UnitPriceWithCommitments",
    "_models.TieredPriceWithCommitments",
    "_models.DynamicPriceWithCommitments",
    "_models.PackagePriceWithCommitments",
]
RateCard = Union["_models.RateCardFlatFee", "_models.RateCardUsageBased"]
RecurringPeriodInterval = Union[str, str, "_models.RecurringPeriodIntervalEnum"]
Entitlement = Union["_models.EntitlementMetered", "_models.EntitlementStatic", "_models.EntitlementBoolean"]
SubscriptionErrorExtensions = "_models.CreateResponseExtensions"
SubscriptionTiming = Union[str, "_models.SubscriptionTimingEnum", datetime.datetime]
SubscriptionEditOperation = Union[
    "_models.EditSubscriptionAddItem",
    "_models.EditSubscriptionRemoveItem",
    "_models.EditSubscriptionAddPhase",
    "_models.EditSubscriptionRemovePhase",
    "_models.EditSubscriptionStretchPhase",
    "_models.EditSubscriptionUnscheduleEdit",
]
MeasureUsageFrom = Union[str, "_models.MeasureUsageFromPreset", datetime.datetime]
App = Union["_models.StripeApp", "_models.SandboxApp", "_models.CustomInvoicingApp"]
NotificationChannel = "_models.NotificationChannelWebhook"
NotificationRule = Union[
    "_models.NotificationRuleBalanceThreshold",
    "_models.NotificationRuleEntitlementReset",
    "_models.NotificationRuleInvoiceCreated",
    "_models.NotificationRuleInvoiceUpdated",
]
InvoiceDocumentRef = "_models.CreditNoteOriginalInvoiceRef"
BillingProfileAppsOrReference = Union["_models.BillingProfileApps", "_models.BillingProfileAppReferences"]
BillingWorkflowCollectionAlignment = Union[
    "_models.BillingWorkflowCollectionAlignmentSubscription", "_models.BillingWorkflowCollectionAlignmentAnchored"
]
BillingDiscountReason = Union[
    "_models.DiscountReasonMaximumSpend",
    "_models.DiscountReasonRatecardPercentage",
    "_models.DiscountReasonRatecardUsage",
]
PaymentTerms = Union["_models.PaymentTermInstant", "_models.PaymentTermDueDate"]
NotificationEventPayload = Union[
    "_models.NotificationEventResetPayload",
    "_models.NotificationEventBalanceThresholdPayload",
    "_models.NotificationEventInvoiceCreatedPayload",
    "_models.NotificationEventInvoiceUpdatedPayload",
]
EntitlementV2 = Union["_models.EntitlementMeteredV2", "_models.EntitlementStaticV2", "_models.EntitlementBooleanV2"]
VoidInvoiceLineAction = Union["_models.VoidInvoiceLineDiscardAction", "_models.VoidInvoiceLinePendingAction"]
AppReplaceUpdate = Union[
    "_models.StripeAppReplaceUpdate", "_models.SandboxAppReplaceUpdate", "_models.CustomInvoicingAppReplaceUpdate"
]
ULIDOrExternalKey = str
ListFeaturesResult = Union[list["_models.Feature"], "_models.FeaturePaginatedResponse"]
SubscriptionCreate = Union["_models.PlanSubscriptionCreate", "_models.CustomSubscriptionCreate"]
SubscriptionChange = Union["_models.PlanSubscriptionChange", "_models.CustomSubscriptionChange"]
ListEntitlementsResult = Union[list["_types.Entitlement"], "_models.EntitlementPaginatedResponse"]
EntitlementCreateInputs = Union[
    "_models.EntitlementMeteredCreateInputs",
    "_models.EntitlementStaticCreateInputs",
    "_models.EntitlementBooleanCreateInputs",
]
IngestEventsBody = Union["_models.Event", list["_models.Event"]]
NotificationChannelCreateRequest = "_models.NotificationChannelWebhookCreateRequest"
NotificationRuleCreateRequest = Union[
    "_models.NotificationRuleBalanceThresholdCreateRequest",
    "_models.NotificationRuleEntitlementResetCreateRequest",
    "_models.NotificationRuleInvoiceCreatedCreateRequest",
    "_models.NotificationRuleInvoiceUpdatedCreateRequest",
]
EntitlementV2CreateInputs = Union[
    "_models.EntitlementMeteredV2CreateInputs",
    "_models.EntitlementStaticCreateInputs",
    "_models.EntitlementBooleanCreateInputs",
]



# coding=utf-8

from copy import deepcopy
from typing import Any, Awaitable
from typing_extensions import Self

from corehttp.rest import AsyncHttpResponse, HttpRequest
from corehttp.runtime import AsyncPipelineClient, policies

from .._utils.serialization import Deserializer, Serializer
from ._configuration import OpenMeterClientConfiguration
from .operations import (
    AddonsOperations,
    AppCustomInvoicingOperations,
    AppStripeOperations,
    AppsOperations,
    BillingProfilesOperations,
    CurrenciesOperations,
    CustomerAppsOperations,
    CustomerEntitlementOperations,
    CustomerEntitlementV2Operations,
    CustomerEntitlementsV2Operations,
    CustomerInvoiceOperations,
    CustomerOperations,
    CustomerOverridesOperations,
    CustomerStripeOperations,
    CustomersOperations,
    DebugOperations,
    EntitlementsOperations,
    EntitlementsV2Operations,
    EventsOperations,
    EventsV2Operations,
    FeaturesOperations,
    GrantsOperations,
    GrantsV2Operations,
    InvoiceOperations,
    InvoicesOperations,
    MarketplaceOperations,
    MetersOperations,
    NotificationChannelsOperations,
    NotificationEventsOperations,
    NotificationRulesOperations,
    PlanAddonsOperations,
    PlansOperations,
    PortalOperations,
    ProgressOperations,
    SubjectsOperations,
    SubscriptionAddonsOperations,
    SubscriptionsOperations,
)


class OpenMeterClient:  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """OpenMeter is a cloud native usage metering service. The OpenMeter API allows you to ingest
    events, query meter usage, and manage resources.

    :ivar portal: PortalOperations operations
    :vartype portal: openmeter.aio.operations.PortalOperations
    :ivar apps: AppsOperations operations
    :vartype apps: openmeter.aio.operations.AppsOperations
    :ivar app_stripe: AppStripeOperations operations
    :vartype app_stripe: openmeter.aio.operations.AppStripeOperations
    :ivar customer_apps: CustomerAppsOperations operations
    :vartype customer_apps: openmeter.aio.operations.CustomerAppsOperations
    :ivar customers: CustomersOperations operations
    :vartype customers: openmeter.aio.operations.CustomersOperations
    :ivar features: FeaturesOperations operations
    :vartype features: openmeter.aio.operations.FeaturesOperations
    :ivar plans: PlansOperations operations
    :vartype plans: openmeter.aio.operations.PlansOperations
    :ivar plan_addons: PlanAddonsOperations operations
    :vartype plan_addons: openmeter.aio.operations.PlanAddonsOperations
    :ivar addons: AddonsOperations operations
    :vartype addons: openmeter.aio.operations.AddonsOperations
    :ivar subscriptions: SubscriptionsOperations operations
    :vartype subscriptions: openmeter.aio.operations.SubscriptionsOperations
    :ivar subscription_addons: SubscriptionAddonsOperations operations
    :vartype subscription_addons: openmeter.aio.operations.SubscriptionAddonsOperations
    :ivar entitlements: EntitlementsOperations operations
    :vartype entitlements: openmeter.aio.operations.EntitlementsOperations
    :ivar grants: GrantsOperations operations
    :vartype grants: openmeter.aio.operations.GrantsOperations
    :ivar subjects: SubjectsOperations operations
    :vartype subjects: openmeter.aio.operations.SubjectsOperations
    :ivar customer: CustomerOperations operations
    :vartype customer: openmeter.aio.operations.CustomerOperations
    :ivar customer_entitlement: CustomerEntitlementOperations operations
    :vartype customer_entitlement: openmeter.aio.operations.CustomerEntitlementOperations
    :ivar customer_stripe: CustomerStripeOperations operations
    :vartype customer_stripe: openmeter.aio.operations.CustomerStripeOperations
    :ivar marketplace: MarketplaceOperations operations
    :vartype marketplace: openmeter.aio.operations.MarketplaceOperations
    :ivar app_custom_invoicing: AppCustomInvoicingOperations operations
    :vartype app_custom_invoicing: openmeter.aio.operations.AppCustomInvoicingOperations
    :ivar events: EventsOperations operations
    :vartype events: openmeter.aio.operations.EventsOperations
    :ivar events_v2: EventsV2Operations operations
    :vartype events_v2: openmeter.aio.operations.EventsV2Operations
    :ivar meters: MetersOperations operations
    :vartype meters: openmeter.aio.operations.MetersOperations
    :ivar subjects: SubjectsOperations operations
    :vartype subjects: openmeter.aio.operations.SubjectsOperations
    :ivar debug: DebugOperations operations
    :vartype debug: openmeter.aio.operations.DebugOperations
    :ivar notification_channels: NotificationChannelsOperations operations
    :vartype notification_channels: openmeter.aio.operations.NotificationChannelsOperations
    :ivar notification_rules: NotificationRulesOperations operations
    :vartype notification_rules: openmeter.aio.operations.NotificationRulesOperations
    :ivar notification_events: NotificationEventsOperations operations
    :vartype notification_events: openmeter.aio.operations.NotificationEventsOperations
    :ivar entitlements_v2: EntitlementsV2Operations operations
    :vartype entitlements_v2: openmeter.aio.operations.EntitlementsV2Operations
    :ivar customer_entitlements_v2: CustomerEntitlementsV2Operations operations
    :vartype customer_entitlements_v2: openmeter.aio.operations.CustomerEntitlementsV2Operations
    :ivar customer_entitlement_v2: CustomerEntitlementV2Operations operations
    :vartype customer_entitlement_v2: openmeter.aio.operations.CustomerEntitlementV2Operations
    :ivar grants_v2: GrantsV2Operations operations
    :vartype grants_v2: openmeter.aio.operations.GrantsV2Operations
    :ivar billing_profiles: BillingProfilesOperations operations
    :vartype billing_profiles: openmeter.aio.operations.BillingProfilesOperations
    :ivar customer_overrides: CustomerOverridesOperations operations
    :vartype customer_overrides: openmeter.aio.operations.CustomerOverridesOperations
    :ivar invoices: InvoicesOperations operations
    :vartype invoices: openmeter.aio.operations.InvoicesOperations
    :ivar invoice: InvoiceOperations operations
    :vartype invoice: openmeter.aio.operations.InvoiceOperations
    :ivar customer_invoice: CustomerInvoiceOperations operations
    :vartype customer_invoice: openmeter.aio.operations.CustomerInvoiceOperations
    :ivar progress: ProgressOperations operations
    :vartype progress: openmeter.aio.operations.ProgressOperations
    :ivar currencies: CurrenciesOperations operations
    :vartype currencies: openmeter.aio.operations.CurrenciesOperations
    :keyword endpoint: Service host. Default value is "https://127.0.0.1".
    :paramtype endpoint: str
    """

    def __init__(  # pylint: disable=missing-client-constructor-parameter-credential
        self, *, endpoint: str = "https://127.0.0.1", **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}"
        self._config = OpenMeterClientConfiguration(endpoint=endpoint, **kwargs)

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.logging_policy,
            ]
        self._client: AsyncPipelineClient = AsyncPipelineClient(endpoint=_endpoint, policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.portal = PortalOperations(self._client, self._config, self._serialize, self._deserialize)
        self.apps = AppsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.app_stripe = AppStripeOperations(self._client, self._config, self._serialize, self._deserialize)
        self.customer_apps = CustomerAppsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.customers = CustomersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.features = FeaturesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.plans = PlansOperations(self._client, self._config, self._serialize, self._deserialize)
        self.plan_addons = PlanAddonsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.addons = AddonsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.subscriptions = SubscriptionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.subscription_addons = SubscriptionAddonsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.entitlements = EntitlementsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.grants = GrantsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.subjects = SubjectsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.customer = CustomerOperations(self._client, self._config, self._serialize, self._deserialize)
        self.customer_entitlement = CustomerEntitlementOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.customer_stripe = CustomerStripeOperations(self._client, self._config, self._serialize, self._deserialize)
        self.marketplace = MarketplaceOperations(self._client, self._config, self._serialize, self._deserialize)
        self.app_custom_invoicing = AppCustomInvoicingOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.events = EventsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.events_v2 = EventsV2Operations(self._client, self._config, self._serialize, self._deserialize)
        self.meters = MetersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.subjects = SubjectsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.debug = DebugOperations(self._client, self._config, self._serialize, self._deserialize)
        self.notification_channels = NotificationChannelsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.notification_rules = NotificationRulesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.notification_events = NotificationEventsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.entitlements_v2 = EntitlementsV2Operations(self._client, self._config, self._serialize, self._deserialize)
        self.customer_entitlements_v2 = CustomerEntitlementsV2Operations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.customer_entitlement_v2 = CustomerEntitlementV2Operations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.grants_v2 = GrantsV2Operations(self._client, self._config, self._serialize, self._deserialize)
        self.billing_profiles = BillingProfilesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.customer_overrides = CustomerOverridesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.invoices = InvoicesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.invoice = InvoiceOperations(self._client, self._config, self._serialize, self._deserialize)
        self.customer_invoice = CustomerInvoiceOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.progress = ProgressOperations(self._client, self._config, self._serialize, self._deserialize)
        self.currencies = CurrenciesOperations(self._client, self._config, self._serialize, self._deserialize)

    def send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from corehttp.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client.send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~corehttp.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~corehttp.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> Self:
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)
